import random
import json
import time
import math
import os # Import os for path joining
from datetime import datetime, timedelta
from flask import Blueprint, jsonify, render_template, request, current_app, flash
# Import the new model
from ..models import VolumeConfig, ActivityLog, PaperMetadata, ActivityCategory, ScheduleConfig, ScraperState, DownloadPathConfig, ScraperModuleConfig
from ..db import db
from ..celery import celery
from ..defaults import MAX_VOLUME
from ..cache_utils import get_cached_hourly_quota, invalidate_hourly_quota_cache
from celery.schedules import crontab
from sqlalchemy import func
from scipaperloader.scrapers.factory import get_scraper, get_available_scrapers

bp = Blueprint("scraper", __name__, url_prefix="/scraper")

# Track the periodic task ID for proper revocation
PERIODIC_TASK_ID = None

# Setup periodic task to run every minute for testing purposes
@celery.on_after_configure.connect
def setup_periodic_tasks(sender, **kwargs):
    global PERIODIC_TASK_ID
    # Run the dummy scraper every minute for testing purposes with a unique task name
    # Using a constant name allows us to revoke this specific periodic task when stopping
    result = sender.add_periodic_task(60.0, run_periodic_dummy_scraper.s(),
                             name='run_dummy_scraper_every_minute')
    PERIODIC_TASK_ID = result.id
    # Log that we've registered the periodic task with its ID
    ActivityLog.log_scraper_command(
        action="register_periodic_task",
        status="success",
        description=f"Registered periodic scraper task with ID: {PERIODIC_TASK_ID}"
    )

# Function to revoke the periodic task properly
def revoke_periodic_task():
    global PERIODIC_TASK_ID
    if PERIODIC_TASK_ID:
        try:
            # Revoke by ID is more reliable than by name
            celery.control.revoke(PERIODIC_TASK_ID, terminate=True)
            
            # Attempt to revoke by name as a fallback (standard Celery method)
            # Note: We're removing the non-standard revoke_from_control method
            # and replacing it with a more reliable approach
            i = celery.control.inspect()
            scheduled = i.scheduled() or {}
            
            # Look for our periodic task by name in the scheduled tasks
            for worker, tasks in scheduled.items():
                for task in tasks:
                    if task.get('name') == 'run_dummy_scraper_every_minute':
                        celery.control.revoke(task['id'], terminate=True)
            
            # Log the action
            ActivityLog.log_scraper_command(
                action="revoke_periodic_task",
                status="success", 
                description=f"Revoked periodic task with ID: {PERIODIC_TASK_ID}"
            )
            return True
        except Exception as e:
            ActivityLog.log_error(
                error_message=f"Failed to revoke periodic task: {str(e)}",
                source="revoke_periodic_task"
            )
            return False
    return False


@celery.task
def run_periodic_dummy_scraper():
    """Periodic task to run the dummy scraper if it's active and not paused"""
    scraper_state = ScraperState.get_current_state()
    # Log every time this runs to track execution
    ActivityLog.log_scraper_activity(
        action="periodic_check",
        status="info",
        description=f"Periodic check running. Scraper active: {scraper_state.is_active}, paused: {scraper_state.is_paused}"
    )
    
    if scraper_state.is_active and not scraper_state.is_paused:
        dummy_scheduled_scraper.delay()
        return True
    return False


@bp.route("/")
def index():
    """Render the scraper control panel."""
    volume_config = VolumeConfig.query.first()

    # Ensure we have volume config
    if not volume_config:
        volume_config = VolumeConfig(volume=100)  # Default value
        db.session.add(volume_config)
        db.session.commit()

    # Get scraper state
    scraper_state = ScraperState.get_current_state()

    return render_template(
        "scraper.html.jinja",
        volume_config=volume_config,
        scraper_active=scraper_state.is_active,
        scraper_paused=scraper_state.is_paused,
        max_volume=MAX_VOLUME
    )


@bp.route("/start", methods=["POST"])
def start_scraper():
    """Start the scraper."""
    scraper_state = ScraperState.get_current_state()

    if not scraper_state.is_active:
        # Update scraper state
        ScraperState.set_active(True)
        ScraperState.set_paused(False)

        # Log the action
        ActivityLog.log_scraper_command(
            action="start_scraper",
            status="success",
            description="Scheduled scraper started - will follow hourly configuration"
        )

        # Immediately trigger a task to test the scheduler and provide feedback
        dummy_scheduled_scraper.delay()

        return jsonify({
            "success": True,
            "message": "Scraper started - following hourly schedule configuration"
        })
    else:
        return jsonify({
            "success": False,
            "message": "Scraper is already running"
        })


@bp.route("/stop", methods=["POST"])
def stop_scraper():
    """Stop the scraper completely."""
    scraper_state = ScraperState.get_current_state()

    if scraper_state.is_active:
        # Update scraper state first
        ScraperState.set_active(False)
        ScraperState.set_paused(False)

        # Stop any running tasks
        task_types_to_revoke = [
            'scipaperloader.blueprints.scraper.process_paper',
            'scipaperloader.blueprints.scraper.dummy_scheduled_scraper',
            'scipaperloader.blueprints.scraper.run_periodic_dummy_scraper'
        ]
        
        # Use our utility function to revoke tasks
        revoked_count = revoke_tasks_by_type(task_types_to_revoke, terminate=True)

        # Revoke the periodic task specifically
        revoke_periodic_task()
        
        # Clear all pending tasks from the queue more aggressively
        try:
            # This purges all tasks in all queues
            celery.control.purge()
            ActivityLog.log_scraper_command(
                action="purge_queue",
                status="success",
                description="Purged all task queues"
            )
        except Exception as e:
            ActivityLog.log_error(
                error_message=f"Failed to purge task queues: {str(e)}",
                source="stop_scraper"
            )
            # Fallback to discard_all if purge fails
            celery.control.discard_all()

        # Restart the worker to ensure clean state
        try:
            celery.control.broadcast('pool_restart', arguments={'reload': True})
            ActivityLog.log_scraper_command(
                action="restart_worker",
                status="success", 
                description="Worker pool restart requested"
            )
        except Exception as e:
            ActivityLog.log_error(
                error_message=f"Failed to restart worker pool: {str(e)}",
                source="stop_scraper"
            )

        ActivityLog.log_scraper_command(
            action="stop_scraper",
            status="success",
            description=f"Scraper stopped manually. Revoked {revoked_count} pending tasks. Worker pool restart requested."
        )

        return jsonify({
            "success": True,
            "message": f"Scraper stopped. Revoked {revoked_count} pending tasks and requested worker restart."
        })
    else:
        return jsonify({
            "success": False,
            "message": "Scraper is not running"
        })


@bp.route("/pause", methods=["POST"])
def pause_scraper():
    """Pause the scraper."""
    scraper_state = ScraperState.get_current_state()

    if scraper_state.is_active and not scraper_state.is_paused:
        # Update scraper state
        ScraperState.set_paused(True)

        # Just revoke processing tasks, but leave the periodic tasks running
        # so it can continue to check the state (which is now paused)
        task_types_to_revoke = [
            'scipaperloader.blueprints.scraper.process_paper',
            'scipaperloader.blueprints.scraper.dummy_scheduled_scraper'
        ]

        # Use our utility function to revoke tasks
        revoked_count = revoke_tasks_by_type(task_types_to_revoke, terminate=True)

        # Also clear the queue
        celery.control.discard_all()

        ActivityLog.log_scraper_command(
            action="pause_scraper",
            status="success",
            description=f"Scraper paused manually. Revoked {revoked_count} pending tasks."
        )

        return jsonify({
            "success": True,
            "message": f"Scraper paused. Revoked {revoked_count} pending tasks."
        })
    elif scraper_state.is_active and scraper_state.is_paused:
        # Update scraper state
        ScraperState.set_paused(False)

        ActivityLog.log_scraper_command(
            action="resume_scraper",
            status="success",
            description="Scraper resumed manually"
        )

        return jsonify({
            "success": True,
            "message": "Scraper resumed"
        })
    else:
        return jsonify({
            "success": False,
            "message": "Scraper is not running"
        })


@bp.route("/status")
def scraper_status():
    """Get the current status of the scraper."""
    scraper_state = ScraperState.get_current_state()

    return jsonify({
        "active": scraper_state.is_active,
        "paused": scraper_state.is_paused,
        "current_hour": datetime.now().hour,
    })


@bp.route("/stats")
def scraper_stats():
    """Get scraper statistics for the dashboard."""
    # Get the last 24 hours of activity
    hours = 24
    if request.args.get('hours'):
        try:
            hours = int(request.args.get('hours'))
        except ValueError:
            pass

    current_time = datetime.utcnow()
    # Use timedelta for proper date calculation instead of simple hour subtraction
    cutoff_time = (current_time - timedelta(hours=hours)).replace(
        minute=0, second=0, microsecond=0
    )

    # Get activity logs for scraper actions
    logs = ActivityLog.query.filter(
        ActivityLog.category == ActivityCategory.SCRAPER_ACTIVITY.value,
        ActivityLog.timestamp >= cutoff_time
    ).all()

    # Group by hour and status
    stats = {}
    for hour in range(hours):
        # Calculate the hour as offset from current time
        target_hour = (current_time.hour - hour) % 24
        stats[target_hour] = {
            "success": 0,
            "error": 0,
            "pending": 0,
            "hour": target_hour,
        }

    for log in logs:
        hour = log.timestamp.hour
        if hour in stats:
            if log.status == "success":
                stats[hour]["success"] += 1
            elif log.status == "error":
                stats[hour]["error"] += 1
            elif log.status == "pending":
                stats[hour]["pending"] += 1

    # Convert to list for easier consumption by JavaScript
    result = [stats[hour] for hour in sorted(stats.keys())]

    return jsonify(result)


@bp.route("/update_config", methods=["POST"])
def update_config():
    """Update scraper configuration."""
    data = request.json

    try:
        if "volume" in data:
            try:
                new_volume = float(data["volume"])

                # Validate volume value
                if new_volume <= 0 or new_volume > MAX_VOLUME:
                    return jsonify({
                        "success": False,
                        "message": f"Volume must be between 1 and {MAX_VOLUME}"
                    })

                volume_config = VolumeConfig.query.first()
                if not volume_config:
                    volume_config = VolumeConfig(volume=new_volume)
                    db.session.add(volume_config)
                else:
                    old_value = volume_config.volume
                    volume_config.volume = new_volume
                    ActivityLog.log_config_change(
                        config_key="scraper_volume",
                        old_value=old_value,
                        new_value=new_volume,
                        description="Updated scraper volume"
                    )

                    # Invalidate hourly quota cache when volume changes
                    invalidate_hourly_quota_cache()

                db.session.commit()
            except (ValueError, TypeError):
                return jsonify({
                    "success": False,
                    "message": "Invalid volume value"
                })

        return jsonify({"success": True, "message": "Configuration updated"})

    except Exception as e:
        db.session.rollback()
        return jsonify({"success": False, "message": f"Unexpected error: {str(e)}"})





@celery.task
def calculate_papers_for_current_hour():
    """
    Calculate how many papers should be downloaded in the current hour
    based on schedule configuration.

    Returns:
        int: Number of papers to download this hour
    """
    current_hour = datetime.now().hour

    # Get volume configuration
    volume_config = VolumeConfig.query.first()
    if not volume_config:
        volume_config = VolumeConfig(volume=100)  # Default to 100 papers per day
        db.session.add(volume_config)
        db.session.commit()

    # Get all schedule configurations to calculate total weight
    schedule_configs = ScheduleConfig.query.all()
    if not schedule_configs:
        # If no schedule configs, create default with equal weights
        for hour in range(24):
            config = ScheduleConfig(hour=hour, weight=1.0)
            db.session.add(config)
        db.session.commit()
        schedule_configs = ScheduleConfig.query.all()

    # Calculate total weight across all hours
    total_weight = sum(config.weight for config in schedule_configs)

    # Find the weight for the current hour
    current_hour_config = ScheduleConfig.query.get(current_hour)
    if not current_hour_config:
        # Create config for current hour if it doesn't exist
        current_hour_config = ScheduleConfig(hour=current_hour, weight=1.0)
        db.session.add(current_hour_config)
        db.session.commit()

    # Calculate papers for current hour: (hour_weight / total_weight) * daily_volume
    if total_weight > 0:
        weight_ratio = current_hour_config.weight / total_weight
        papers_this_hour = math.floor(weight_ratio * volume_config.volume)
    else:
        papers_this_hour = 0

    return papers_this_hour


@celery.task
def dummy_scheduled_scraper():
    """
    Selects new papers based on the hourly schedule and marks them as Pending.
    Then schedules their processing randomly within the hour.
    """
    scraper_state = ScraperState.get_current_state()
    if not scraper_state.is_active or scraper_state.is_paused:
        ActivityLog.log_scraper_activity(
            action="dummy_scheduled_scraper_skip",
            status="info",
            description="Skipping run because scraper is inactive or paused."
        )
        return False # Stop if not active/paused

    # Use cached hourly quota instead of calculating each time
    papers_to_select = get_cached_hourly_quota(calculate_papers_for_current_hour)

    if papers_to_select <= 0:
        ActivityLog.log_scraper_activity(
            action="dummy_scheduled_scraper_info",
            status="info",
            description=f"Hourly quota is {papers_to_select}. No papers to select this hour."
        )
        return True # Nothing to do this hour based on schedule

    # --- Core Logic Change: Select NEW papers ---
    try:
        # Find "New" papers, select randomly up to the calculated limit
        new_papers = PaperMetadata.query.filter_by(status="New") \
                                        .order_by(func.random()) \
                                        .limit(papers_to_select) \
                                        .all()

        if not new_papers:
            ActivityLog.log_scraper_activity(
                action="dummy_scheduled_scraper_info",
                status="info",
                description="No 'New' papers found in the database. Stopping scraper."
            )
            
            # Stop the scraper since there are no more papers to process
            ScraperState.set_active(False)
            ActivityLog.log_scraper_command(
                action="auto_stop_scraper",
                status="success",
                description="Scraper automatically stopped due to no 'New' papers left to process."
            )
            
            return True

        selected_paper_ids = [p.id for p in new_papers]
        
        # Update status to "Pending" in bulk for efficiency
        PaperMetadata.query.filter(PaperMetadata.id.in_(selected_paper_ids)) \
                       .update({"status": "Pending", "updated_at": datetime.utcnow()}, synchronize_session=False)
        db.session.commit()

        ActivityLog.log_scraper_activity(
            action="select_new_papers",
            status="success",
            description=f"Selected {len(selected_paper_ids)} 'New' papers and marked as 'Pending'. IDs: {selected_paper_ids}"
        )

        # --- Now schedule processing for the newly selected "Pending" papers ---
        # (Using the new modular process_paper task)
        # Add random delays for processing within the hour (e.g., up to 3600 seconds)
        for paper_id in selected_paper_ids:
            delay = random.uniform(1, 3500) # Random delay up to ~58 minutes
            process_paper.apply_async(args=[paper_id], countdown=delay)
            
        ActivityLog.log_scraper_activity(
            action="schedule_processing",
            status="success",
            description=f"Scheduled processing for {len(selected_paper_ids)} papers with random delays."
        )
        
        return True

    except Exception as e:
        db.session.rollback() # Rollback DB changes on error
        ActivityLog.log_error(
            error_message=f"Error in dummy_scheduled_scraper: {str(e)}",
            source="dummy_scheduled_scraper"
        )
        return False


@celery.task(bind=True)
def process_paper_batch(self, paper_ids):
    """
    Process a batch of papers to improve throughput and reduce overhead.
    
    Args:
        paper_ids (list): List of paper IDs to process in this batch
    """
    # Check if scraper is still active
    scraper_state = ScraperState.get_current_state()
    if not scraper_state.is_active or scraper_state.is_paused:
        ActivityLog.log_scraper_activity(
            action="process_paper_batch",
            status="info",
            description=f"Skipped batch of {len(paper_ids)} papers because scraper is {'paused' if scraper_state.is_paused else 'stopped'}"
        )
        return False
    
    # Log the batch starting
    ActivityLog.log_scraper_activity(
        action="process_paper_batch",
        status="info",
        description=f"Started processing batch of {len(paper_ids)} papers"
    )
    
    # Process each paper in the batch
    results = {
        "success": 0,
        "failure": 0,
        "skipped": 0
    }
    
    # Begin a transaction for the entire batch
    try:
        for paper_id in paper_ids:
            # Double-check scraper state before each paper
            scraper_state = ScraperState.get_current_state()
            if not scraper_state.is_active or scraper_state.is_paused:
                results["skipped"] += 1
                continue
                
            # Get the paper from database
            paper = PaperMetadata.query.get(paper_id)
            if not paper:
                results["skipped"] += 1
                continue
                
            # Simulate random success/failure (70% success rate)
            success = random.random() < 0.7
            
            # Simulate some processing time (0.5-2 seconds per paper in batch)
            time.sleep(random.uniform(0.5, 2))
            
            if success:
                # Update paper status to "Done"
                paper.status = "Done"
                paper.file_path = f"/path/to/dummy/papers/{paper.doi.replace('/', '_')}.pdf"
                results["success"] += 1
                
                # Log individual paper success with minimal overhead
                ActivityLog.log_scraper_activity(
                    action="process_paper",
                    paper_id=paper.id,
                    status="success",
                    description=f"Processed in batch: {paper.title}"
                )
            else:
                # Update paper status to "Failed"
                paper.status = "Failed"
                
                # Generate random error message
                error_message = random.choice([
                    "Publisher website unavailable",
                    "No PDF download link found",
                    "Access restricted",
                    "Download timeout",
                    "Invalid DOI",
                    "Rate limited by publisher"
                ])
                paper.error_msg = error_message
                results["failure"] += 1
                
                # Log individual paper failure with minimal overhead
                ActivityLog.log_scraper_activity(
                    action="process_paper",
                    paper_id=paper.id,
                    status="error",
                    description=f"Failed in batch: {error_message}"
                )
                
            # Update the timestamp
            paper.updated_at = datetime.utcnow()
        
        # Commit the entire batch at once
        db.session.commit()
        
    except Exception as e:
        # If any error occurs, roll back the entire batch
        db.session.rollback()
        ActivityLog.log_error(
            error_message=f"Error processing paper batch: {str(e)}",
            source="process_paper_batch"
        )
        return False
    
    # Log batch completion
    ActivityLog.log_scraper_activity(
        action="process_paper_batch",
        status="success",
        description=f"Completed batch processing: {results['success']} succeeded, {results['failure']} failed, {results['skipped']} skipped"
    )
    
    return results


@bp.route("/reset", methods=["POST"])
def reset_scraper():
    """
    Reset the scraper completely:
    1. Stop all running tasks
    2. Optionally purge all papers except those with 'Pending' status
    3. Reset scraper state to active and unpaused
    4. Trigger a new scraping cycle
    """
    # First stop everything
    stop_scraper()
    
    # Check if we should clear papers
    clear_papers = request.json.get('clear_papers', True) if request.is_json else True
    
    if clear_papers:
        try:
            # Get all papers that aren't in Pending status
            papers = PaperMetadata.query.filter(PaperMetadata.status != "Pending").all()
            count = len(papers)
            
            # Delete them all
            for paper in papers:
                db.session.delete(paper)
            
            db.session.commit()
            
            ActivityLog.log_scraper_command(
                action="reset_scraper",
                status="success",
                description=f"Reset scraper and cleared {count} non-pending papers"
            )
        except Exception as e:
            db.session.rollback()
            ActivityLog.log_error(
                error_message=f"Failed to reset papers: {str(e)}",
                source="reset_scraper"
            )
            return jsonify({
                "success": False,
                "message": f"Error clearing papers: {str(e)}"
            })
    
    # Set state to active and unpaused
    ScraperState.set_active(True)
    ScraperState.set_paused(False)
    
    # Re-register the periodic task if needed
    setup_periodic_tasks(celery)
    
    # Kick off a fresh scraping cycle
    dummy_scheduled_scraper.delay()
    
    return jsonify({
        "success": True,
        "message": "Scraper has been completely reset and restarted"
    })

# Common utility function to revoke tasks by type
def revoke_tasks_by_type(task_types, terminate=True):
    """
    Revokes all tasks of specified types across scheduled, reserved and active queues.
    
    Args:
        task_types (list): List of task name strings to revoke
        terminate (bool): Whether to terminate running tasks
        
    Returns:
        int: Count of revoked tasks
    """
    # Get all tasks of all types
    i = celery.control.inspect()
    scheduled = i.scheduled() or {}
    reserved = i.reserved() or {}
    active = i.active() or {}
    
    revoked_count = 0
    
    # Revoke all scheduled tasks
    for worker, tasks in scheduled.items():
        for task in tasks:
            if task['name'] in task_types:
                celery.control.revoke(task['id'], terminate=terminate)
                revoked_count += 1
    
    # Revoke all reserved tasks
    for worker, tasks in reserved.items():
        for task in tasks:
            if task['name'] in task_types:
                celery.control.revoke(task['id'], terminate=terminate)
                revoked_count += 1
    
    # Revoke all active tasks
    for worker, tasks in active.items():
        for task in tasks:
            if task['name'] in task_types:
                celery.control.revoke(task['id'], terminate=terminate)
                revoked_count += 1
    
    return revoked_count

import math
from datetime import datetime
from ..models import VolumeConfig, ScheduleConfig, PaperMetadata # Ensure imports
from ..db import db # Ensure import

def calculate_papers_for_current_hour():
    """Calculates the target number of papers for the current hour using w*N/24."""
    current_hour = datetime.now().hour
    volume_config = VolumeConfig.query.first()
    # Use default if not set
    volume = volume_config.volume if volume_config else 100 

    current_hour_config = ScheduleConfig.query.filter_by(hour=current_hour).first()
    # Use default weight 1.0 if not set for the hour
    weight = current_hour_config.weight if current_hour_config else 1.0 

    # Calculate papers using the formula: w * N / 24
    papers_this_hour = math.floor(weight * volume / 24)
    
    # Log the calculation for debugging
    ActivityLog.log_scraper_activity(
        action="calculate_hourly_quota",
        status="info",
        description=f"Hour {current_hour}: weight={weight:.2f}, total_volume={volume}, target_papers={papers_this_hour}"
    )
    
    return papers_this_hour


@celery.task(bind=True)
def process_paper(self, paper_id):
    """Process a paper using the configured scraper."""
    from scipaperloader.models import PaperMetadata
    paper = PaperMetadata.query.get(paper_id)
    if not paper:
        return {"status": "error", "message": f"Paper with ID {paper_id} not found"}

    scraper = get_scraper()
    result = scraper.scrape(paper.doi)

    return {
        "paper_id": paper_id,
        "status": result.status,
        "message": result.message
    }


@celery.task(bind=True)
@celery.task(bind=True)
def process_paper_with_scraper(self, paper_id, scraper_module):
    """Process a paper using a specific scraper module."""
    from scipaperloader.models import PaperMetadata
    import importlib
    from ..scrapers.base import BaseScraper
    
    paper = PaperMetadata.query.get(paper_id)
    if not paper:
        return {"status": "error", "message": f"Paper with ID {paper_id} not found"}

    try:
        # Import the specified scraper module
        module = importlib.import_module(f"scipaperloader.scrapers.{scraper_module}")
        cls = getattr(module, "Scraper")
        
        # Validate that it's a BaseScraper
        if not issubclass(cls, BaseScraper):
            error_msg = f"Scraper class in module '{scraper_module}' does not inherit from BaseScraper"
            ActivityLog.log_error(
                error_message=error_msg,
                source="process_paper_with_scraper"
            )
            return {"status": "error", "message": error_msg}
            
        # Instantiate and use the scraper
        scraper = cls()
        result = scraper.scrape(paper.doi)
        
        return {
            "paper_id": paper_id,
            "status": result.status,
            "message": result.message,
            "scraper": scraper_module
        }
    
    except (ImportError, AttributeError) as e:
        error_msg = f"Failed to load scraper module '{scraper_module}': {str(e)}"
        ActivityLog.log_error(
            error_message=error_msg,
            source="process_paper_with_scraper"
        )
        return {"status": "error", "message": error_msg}
    except Exception as e:
        error_msg = f"Error processing paper with scraper '{scraper_module}': {str(e)}"
        ActivityLog.log_error(
            error_message=error_msg,
            source="process_paper_with_scraper",
            exception=e
        )
        return {"status": "error", "message": error_msg}


@bp.route("/process_single/<int:paper_id>", methods=["POST"])
def process_single_paper(paper_id):
    """Process a single paper by ID."""
    try:
        # Check if paper exists
        paper = PaperMetadata.query.get(paper_id)
        if not paper:
            return jsonify({
                "success": False,
                "message": f"Paper with ID {paper_id} not found"
            })
        
        # Get the scraper module name from the request
        scraper_module = None
        if request.is_json and request.json:
            scraper_module = request.json.get('scraper_module')
        
        # Update status to Pending
        old_status = paper.status
        paper.status = "Pending"
        paper.updated_at = datetime.utcnow()
        db.session.commit()
        
        # Log that we're processing this paper
        ActivityLog.log_scraper_activity(
            action="manual_process_paper",
            paper_id=paper_id,
            status="pending",
            description=f"Manual processing initiated for paper: {paper.title}" + 
                       (f" using {scraper_module} scraper" if scraper_module else "")
        )
        
        # Start the task (without delay since it's manual)
        if scraper_module:
            task = process_paper_with_scraper.delay(paper_id, scraper_module)
        else:
            task = process_paper.delay(paper_id)
        
        return jsonify({
            "success": True,
            "task_id": task.id,
            "message": f"Processing paper '{paper.title}' (ID: {paper_id})" + 
                      (f" using {scraper_module} scraper" if scraper_module else "") +
                      f". Previous status: {old_status}"
        })
        
    except Exception as e:
        db.session.rollback()
        ActivityLog.log_error(
            error_message=f"Failed to process paper {paper_id}: {str(e)}",
            exception=e,
            source="process_single_paper"
        )
        return jsonify({
            "success": False,
            "message": f"Error: {str(e)}"
        })


@bp.route("/available_scrapers")
def available_scrapers():
    """Get list of available scraper modules."""
    from scipaperloader.scrapers.factory import get_available_scrapers
    from ..models import ScraperModuleConfig
    
    try:
        scrapers = get_available_scrapers()
        current_module = ScraperModuleConfig.get_current_module()
        
        return jsonify({
            "success": True,
            "scrapers": [
                {
                    "name": s["name"],
                    "description": s["description"],
                    "is_current": s["name"] == current_module
                } for s in scrapers
            ],
            "current": current_module
        })
        
    except Exception as e:
        ActivityLog.log_error(
            error_message=f"Failed to get available scrapers: {str(e)}",
            source="available_scrapers"
        )
        return jsonify({
            "success": False,
            "message": f"Error: {str(e)}",
            "scrapers": []
        })
